@@ -26,12 +26,12 @@
 #ifndef _COMPIZ_H
 #define _COMPIZ_H
 
-#define ABIVERSION 20070815
+#define ABIVERSION 20070915
 
 #include <stdio.h>
 #include <sys/time.h>
 
-#include <X11/Xlib-xcb.h>
+#include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/Xdamage.h>
 #include <X11/extensions/Xcomposite.h>
@@ -278,10 +278,6 @@ typedef enum {
     CompOptionTypeString,
     CompOptionTypeColor,
     CompOptionTypeAction,
-    CompOptionTypeKey,
-    CompOptionTypeButton,
-    CompOptionTypeEdge,
-    CompOptionTypeBell,
     CompOptionTypeMatch,
     CompOptionTypeList
 } CompOptionType;
@@ -314,6 +310,7 @@ typedef enum {
     CompLogLevelWarn,
     CompLogLevelInfo,
     CompLogLevelDebug
+
 } CompLogLevel;
 
 typedef struct _CompKeyBinding {
@@ -350,6 +347,7 @@ struct _CompAction {
     Bool bell;
 
     unsigned int edgeMask;
+    int		 edgeButton;
 
     CompPrivate priv;
 };
@@ -405,13 +403,6 @@ struct _CompOption {
 typedef CompOption *(*DisplayOptionsProc) (CompDisplay *display, int *count);
 typedef CompOption *(*ScreenOptionsProc) (CompScreen *screen, int *count);
 
-void
-compInitOptionValue (CompOptionValue *v);
-
-void
-compFiniOptionValue (CompOptionValue *v,
-		     CompOptionType  type);
-
 void
 compInitOption (CompOption *option);
 
@@ -421,7 +412,7 @@ compFiniOption (CompOption *option);
 CompOption *
 compFindOption (CompOption *option,
 		int	    nOption,
-		const char  *name,
+		char	    *name,
 		int	    *index);
 
 Bool
@@ -466,37 +457,37 @@ compWindowTypeMaskFromStringList (CompOptionValue *value);
 Bool
 getBoolOptionNamed (CompOption *option,
 		    int	       nOption,
-		    const char *name,
+		    char       *name,
 		    Bool       defaultValue);
 
 int
 getIntOptionNamed (CompOption *option,
 		   int	      nOption,
-		   const char *name,
+		   char	      *name,
 		   int	      defaultValue);
 
 float
 getFloatOptionNamed (CompOption *option,
 		     int	nOption,
-		     const char *name,
+		     char	*name,
 		     float	defaultValue);
 
 char *
 getStringOptionNamed (CompOption *option,
 		      int	 nOption,
-		      const char *name,
+		      char	 *name,
 		      char	 *defaultValue);
 
 unsigned short *
 getColorOptionNamed (CompOption	    *option,
 		     int	    nOption,
-		     const char     *name,
+		     char	    *name,
 		     unsigned short *defaultValue);
 
 CompMatch *
 getMatchOptionNamed (CompOption	*option,
 		     int	nOption,
-		     const char *name,
+		     char	*name,
 		     CompMatch  *defaultValue);
 
 char *
@@ -507,14 +498,6 @@ char *
 buttonBindingToString (CompDisplay       *d,
 		       CompButtonBinding *button);
 
-char *
-keyActionToString (CompDisplay *d,
-		   CompAction  *action);
-
-char *
-buttonActionToString (CompDisplay *d,
-		      CompAction  *action);
-
 Bool
 stringToKeyBinding (CompDisplay    *d,
 		    const char     *binding,
@@ -525,24 +508,8 @@ stringToButtonBinding (CompDisplay	 *d,
 		       const char	 *binding,
 		       CompButtonBinding *button);
 
-void
-stringToKeyAction (CompDisplay *d,
-		   const char  *binding,
-		   CompAction  *action);
-
-void
-stringToButtonAction (CompDisplay *d,
-		      const char  *binding,
-		      CompAction  *action);
-
-const char *
-edgeToString (unsigned int edge);
-
-unsigned int
-stringToEdgeMask (const char *edge);
-
 char *
-edgeMaskToString (unsigned int edgeMask);
+edgeToString (unsigned int edge);
 
 Bool
 stringToColor (const char     *color,
@@ -551,12 +518,9 @@ stringToColor (const char     *color,
 char *
 colorToString (unsigned short *rgba);
 
-const char *
+char *
 optionTypeToString (CompOptionType type);
 
-Bool
-isActionOption (CompOption *option);
-
 
 /* display.c */
 
@@ -564,73 +528,72 @@ typedef int CompTimeoutHandle;
 typedef int CompWatchFdHandle;
 typedef int CompFileWatchHandle;
 
-#define COMP_DISPLAY_OPTION_ACTIVE_PLUGINS                   0
-#define COMP_DISPLAY_OPTION_TEXTURE_FILTER                   1
-#define COMP_DISPLAY_OPTION_CLICK_TO_FOCUS                   2
-#define COMP_DISPLAY_OPTION_AUTORAISE                        3
-#define COMP_DISPLAY_OPTION_AUTORAISE_DELAY                  4
-#define COMP_DISPLAY_OPTION_CLOSE_WINDOW_KEY                 5
-#define COMP_DISPLAY_OPTION_MAIN_MENU_KEY                    6
-#define COMP_DISPLAY_OPTION_RUN_DIALOG_KEY                   7
-#define COMP_DISPLAY_OPTION_COMMAND0                         8
-#define COMP_DISPLAY_OPTION_COMMAND1                         9
-#define COMP_DISPLAY_OPTION_COMMAND2                         10
-#define COMP_DISPLAY_OPTION_COMMAND3                         11
-#define COMP_DISPLAY_OPTION_COMMAND4                         12
-#define COMP_DISPLAY_OPTION_COMMAND5                         13
-#define COMP_DISPLAY_OPTION_COMMAND6                         14
-#define COMP_DISPLAY_OPTION_COMMAND7                         15
-#define COMP_DISPLAY_OPTION_COMMAND8                         16
-#define COMP_DISPLAY_OPTION_COMMAND9                         17
-#define COMP_DISPLAY_OPTION_COMMAND10                        18
-#define COMP_DISPLAY_OPTION_COMMAND11                        19
-#define COMP_DISPLAY_OPTION_RUN_COMMAND0_KEY                 20
-#define COMP_DISPLAY_OPTION_RUN_COMMAND1_KEY                 21
-#define COMP_DISPLAY_OPTION_RUN_COMMAND2_KEY                 22
-#define COMP_DISPLAY_OPTION_RUN_COMMAND3_KEY                 23
-#define COMP_DISPLAY_OPTION_RUN_COMMAND4_KEY                 24
-#define COMP_DISPLAY_OPTION_RUN_COMMAND5_KEY                 25
-#define COMP_DISPLAY_OPTION_RUN_COMMAND6_KEY                 26
-#define COMP_DISPLAY_OPTION_RUN_COMMAND7_KEY                 27
-#define COMP_DISPLAY_OPTION_RUN_COMMAND8_KEY                 28
-#define COMP_DISPLAY_OPTION_RUN_COMMAND9_KEY                 29
-#define COMP_DISPLAY_OPTION_RUN_COMMAND10_KEY                30
-#define COMP_DISPLAY_OPTION_RUN_COMMAND11_KEY                31
-#define COMP_DISPLAY_OPTION_SLOW_ANIMATIONS_KEY              32
-#define COMP_DISPLAY_OPTION_RAISE_WINDOW_BUTTON              33
-#define COMP_DISPLAY_OPTION_LOWER_WINDOW_BUTTON              34
-#define COMP_DISPLAY_OPTION_UNMAXIMIZE_WINDOW_KEY            35
-#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW_KEY              36
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_KEY              37
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_HORZ_KEY         38
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_VERT_KEY         39
-#define COMP_DISPLAY_OPTION_OPACITY_INCREASE_BUTTON          40
-#define COMP_DISPLAY_OPTION_OPACITY_DECREASE_BUTTON          41
-#define COMP_DISPLAY_OPTION_SCREENSHOT                       42
-#define COMP_DISPLAY_OPTION_RUN_SCREENSHOT_KEY               43
-#define COMP_DISPLAY_OPTION_WINDOW_SCREENSHOT                44
-#define COMP_DISPLAY_OPTION_RUN_WINDOW_SCREENSHOT_KEY        45
-#define COMP_DISPLAY_OPTION_WINDOW_MENU_BUTTON               46
-#define COMP_DISPLAY_OPTION_WINDOW_MENU_KEY                  47
-#define COMP_DISPLAY_OPTION_SHOW_DESKTOP_KEY                 48
-#define COMP_DISPLAY_OPTION_RAISE_ON_CLICK                   49
-#define COMP_DISPLAY_OPTION_AUDIBLE_BELL                     50
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_KEY      51
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_HORZ_KEY 52
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_VERT_KEY 53
-#define COMP_DISPLAY_OPTION_HIDE_SKIP_TASKBAR_WINDOWS        54
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_SHADED_KEY         55
-#define COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED      56
-#define COMP_DISPLAY_OPTION_TERMINAL			     57
-#define COMP_DISPLAY_OPTION_RUN_TERMINAL_KEY		     58
-#define COMP_DISPLAY_OPTION_PING_DELAY			     59
-#define COMP_DISPLAY_OPTION_NUM				     60
+#define COMP_DISPLAY_OPTION_ACTIVE_PLUGINS                0
+#define COMP_DISPLAY_OPTION_TEXTURE_FILTER                1
+#define COMP_DISPLAY_OPTION_CLICK_TO_FOCUS                2
+#define COMP_DISPLAY_OPTION_AUTORAISE                     3
+#define COMP_DISPLAY_OPTION_AUTORAISE_DELAY               4
+#define COMP_DISPLAY_OPTION_CLOSE_WINDOW                  5
+#define COMP_DISPLAY_OPTION_MAIN_MENU                     6
+#define COMP_DISPLAY_OPTION_RUN_DIALOG                    7
+#define COMP_DISPLAY_OPTION_COMMAND0                      8
+#define COMP_DISPLAY_OPTION_COMMAND1                      9
+#define COMP_DISPLAY_OPTION_COMMAND2                      10
+#define COMP_DISPLAY_OPTION_COMMAND3                      11
+#define COMP_DISPLAY_OPTION_COMMAND4                      12
+#define COMP_DISPLAY_OPTION_COMMAND5                      13
+#define COMP_DISPLAY_OPTION_COMMAND6                      14
+#define COMP_DISPLAY_OPTION_COMMAND7                      15
+#define COMP_DISPLAY_OPTION_COMMAND8                      16
+#define COMP_DISPLAY_OPTION_COMMAND9                      17
+#define COMP_DISPLAY_OPTION_COMMAND10                     18
+#define COMP_DISPLAY_OPTION_COMMAND11                     19
+#define COMP_DISPLAY_OPTION_RUN_COMMAND0                  20
+#define COMP_DISPLAY_OPTION_RUN_COMMAND1                  21
+#define COMP_DISPLAY_OPTION_RUN_COMMAND2                  22
+#define COMP_DISPLAY_OPTION_RUN_COMMAND3                  23
+#define COMP_DISPLAY_OPTION_RUN_COMMAND4                  24
+#define COMP_DISPLAY_OPTION_RUN_COMMAND5                  25
+#define COMP_DISPLAY_OPTION_RUN_COMMAND6                  26
+#define COMP_DISPLAY_OPTION_RUN_COMMAND7                  27
+#define COMP_DISPLAY_OPTION_RUN_COMMAND8                  28
+#define COMP_DISPLAY_OPTION_RUN_COMMAND9                  29
+#define COMP_DISPLAY_OPTION_RUN_COMMAND10                 30
+#define COMP_DISPLAY_OPTION_RUN_COMMAND11                 31
+#define COMP_DISPLAY_OPTION_SLOW_ANIMATIONS               32
+#define COMP_DISPLAY_OPTION_RAISE_WINDOW                  33
+#define COMP_DISPLAY_OPTION_LOWER_WINDOW                  34
+#define COMP_DISPLAY_OPTION_UNMAXIMIZE_WINDOW             35
+#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW               36
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW               37
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_HORZ          38
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_VERT          39
+#define COMP_DISPLAY_OPTION_OPACITY_INCREASE              40
+#define COMP_DISPLAY_OPTION_OPACITY_DECREASE              41
+#define COMP_DISPLAY_OPTION_SCREENSHOT                    42
+#define COMP_DISPLAY_OPTION_RUN_SCREENSHOT                43
+#define COMP_DISPLAY_OPTION_WINDOW_SCREENSHOT             44
+#define COMP_DISPLAY_OPTION_RUN_WINDOW_SCREENSHOT         45
+#define COMP_DISPLAY_OPTION_WINDOW_MENU                   46
+#define COMP_DISPLAY_OPTION_SHOW_DESKTOP                  47
+#define COMP_DISPLAY_OPTION_RAISE_ON_CLICK                48
+#define COMP_DISPLAY_OPTION_AUDIBLE_BELL                  49
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED       50
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_HORZ  51
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_VERT  52
+#define COMP_DISPLAY_OPTION_HIDE_SKIP_TASKBAR_WINDOWS     53
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_SHADED          54
+#define COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED   55
+#define COMP_DISPLAY_OPTION_TERMINAL			  56
+#define COMP_DISPLAY_OPTION_RUN_TERMINAL		  57
+#define COMP_DISPLAY_OPTION_PING_DELAY			  58
+#define COMP_DISPLAY_OPTION_NUM				  59
 
 typedef CompOption *(*GetDisplayOptionsProc) (CompDisplay *display,
 					      int	  *count);
 
 typedef Bool (*SetDisplayOptionProc) (CompDisplay     *display,
-				      const char      *name,
+				      char	      *name,
 				      CompOptionValue *value);
 
 typedef CompOption *(*GetPluginDisplayOptionsProc) (CompPlugin  *plugin,
@@ -639,12 +602,12 @@ typedef CompOption *(*GetPluginDisplayOptionsProc) (CompPlugin  *plugin,
 
 typedef Bool (*SetPluginDisplayOptionProc) (CompPlugin      *plugin,
 					    CompDisplay     *display,
-					    const char	    *name,
+					    char	    *name,
 					    CompOptionValue *value);
 
 typedef Bool (*SetDisplayOptionForPluginProc) (CompDisplay     *display,
-					       const char      *plugin,
-					       const char	*name,
+					       char	       *plugin,
+					       char	       *name,
 					       CompOptionValue *value);
 
 typedef Bool (*InitPluginForDisplayProc) (CompPlugin  *plugin,
@@ -657,8 +620,8 @@ typedef void (*HandleEventProc) (CompDisplay *display,
 				 XEvent	     *event);
 
 typedef void (*HandleCompizEventProc) (CompDisplay *display,
-				       const char  *pluginName,
-				       const char  *eventName,
+				       char        *pluginName,
+				       char        *eventName,
 				       CompOption  *option,
 				       int         nOption);
 
@@ -764,13 +727,11 @@ typedef void (*MatchPropertyChangedProc) (CompDisplay *display,
 					  CompWindow  *window);
 
 typedef void (*LogMessageProc) (CompDisplay  *d,
-				const char   *componentName,
+				char         *componentName,
 				CompLogLevel level,
-				const char   *message);
+				char         *message);
 
 struct _CompDisplay {
-    xcb_connection_t *connection;
-
     Display    *display;
     CompScreen *screens;
 
@@ -1028,16 +989,16 @@ fileWatchRemoved (CompDisplay   *display,
 
 void
 compLogMessage (CompDisplay  *d,
-		const char   *componentName,
+		char         *componentName,
 		CompLogLevel level,
-		const char   *format,
+		char         *format,
 		...);
 
 void
 logMessage (CompDisplay  *d,
-	    const char   *componentName,
+	    char         *componentName,
 	    CompLogLevel level,
-	    const char   *message);
+	    char         *message);
 
 const char *
 logLevelToString (CompLogLevel level);
@@ -1050,7 +1011,7 @@ addScreenToDisplay (CompDisplay *display,
 		    CompScreen *s);
 
 Bool
-addDisplay (const char *name);
+addDisplay (char *name);
 
 Time
 getCurrentTimeFromDisplay (CompDisplay *d);
@@ -1154,8 +1115,8 @@ handleEvent (CompDisplay *display,
 
 void
 handleCompizEvent (CompDisplay *display,
-		   const char  *pluginName,
-		   const char  *eventName,
+		   char        *pluginName,
+		   char        *eventName,
 		   CompOption  *option,
 		   int         nOption);
 
@@ -1496,14 +1457,14 @@ destroyTexture (CompScreen  *screen,
 Bool
 imageBufferToTexture (CompScreen   *screen,
 		      CompTexture  *texture,
-		      const char   *image,
+		      char	   *image,
 		      unsigned int width,
 		      unsigned int height);
 
 Bool
 imageDataToTexture (CompScreen   *screen,
 		    CompTexture  *texture,
-		    const char	 *image,
+		    char	 *image,
 		    unsigned int width,
 		    unsigned int height,
 		    GLenum       format,
@@ -1675,18 +1636,18 @@ typedef void (*GLGenerateMipmapProc) (GLenum target);
 typedef CompOption *(*GetScreenOptionsProc) (CompScreen *screen,
 					     int	*count);
 typedef Bool (*SetScreenOptionProc) (CompScreen      *screen,
-				     const char	     *name,
+				     char	     *name,
 				     CompOptionValue *value);
 typedef CompOption *(*GetPluginScreenOptionsProc) (CompPlugin *plugin,
 						   CompScreen *screen,
 						   int	      *count);
 typedef Bool (*SetPluginScreenOptionProc) (CompPlugin      *plugin,
 					   CompScreen      *screen,
-					   const char	   *name,
+					   char	           *name,
 					   CompOptionValue *value);
 typedef Bool (*SetScreenOptionForPluginProc) (CompScreen      *screen,
-					      const char      *plugin,
-					      const char      *name,
+					      char	      *plugin,
+					      char	      *name,
 					      CompOptionValue *value);
 
 typedef Bool (*InitPluginForScreenProc) (CompPlugin *plugin,
@@ -1715,7 +1676,9 @@ typedef Bool (*DamageCursorRectProc) (CompCursor *c,
 typedef void (*GetOutputExtentsForWindowProc) (CompWindow	 *w,
 					       CompWindowExtents *output);
 
-typedef unsigned int (*GetAllowedActionsForWindowProc) (CompWindow *w);
+typedef void (*GetAllowedActionsForWindowProc) (CompWindow   *w,
+						unsigned int *setActions,
+						unsigned int *clearActions);
 
 typedef Bool (*FocusWindowProc) (CompWindow *window);
 
@@ -1749,7 +1712,8 @@ typedef void (*WindowGrabNotifyProc) (CompWindow   *window,
 
 typedef void (*WindowUngrabNotifyProc) (CompWindow *window);
 
-typedef void (*WindowStateChangeNotifyProc) (CompWindow *window);
+typedef void (*WindowStateChangeNotifyProc) (CompWindow   *window,
+					     unsigned int lastState);
 
 typedef void (*WindowAddNotifyProc) (CompWindow *window);
 
@@ -2395,7 +2359,6 @@ struct _CompWindow {
     unsigned int      wmType;
     unsigned int      type;
     unsigned int      state;
-    unsigned int      lastState;
     unsigned int      actions;
     unsigned int      protocols;
     unsigned int      mwmDecor;
@@ -2696,8 +2659,10 @@ void
 getOutputExtentsForWindow (CompWindow	     *w,
 			   CompWindowExtents *output);
 
-unsigned int
-getAllowedActionsForWindow (CompWindow *w);
+void
+getAllowedActionsForWindow (CompWindow   *w,
+			    unsigned int *setActions,
+			    unsigned int *clearActions);
 
 void
 addWindowDamage (CompWindow *w);
@@ -2752,7 +2717,8 @@ void
 windowUngrabNotify (CompWindow *w);
 
 void
-windowStateChangeNotify (CompWindow *w);
+windowStateChangeNotify (CompWindow   *w,
+			 unsigned int lastState);
 
 void
 moveInputFocusToWindow (CompWindow *w);
@@ -2817,7 +2783,7 @@ setWindowUserTime (CompWindow *w,
 		   Time       time);
 
 Bool
-focusWindowOnMap (CompWindow *w);
+allowWindowFocus (CompWindow *w);
 
 void
 unredirectWindow (CompWindow *w);
@@ -2869,7 +2835,7 @@ typedef void (*FiniPluginProc) (CompPlugin *plugin);
 typedef CompMetadata *(*GetMetadataProc) (CompPlugin *plugin);
 
 typedef struct _CompPluginVTable {
-    const char *name;
+    char *name;
 
     GetVersionProc  getVersion;
     GetMetadataProc getMetadata;
@@ -2895,13 +2861,13 @@ typedef struct _CompPluginVTable {
 typedef CompPluginVTable *(*PluginGetInfoProc) (void);
 
 typedef Bool (*LoadPluginProc) (CompPlugin *p,
-				const char *path,
-				const char *name);
+				char	   *path,
+				char	   *name);
 
 typedef void (*UnloadPluginProc) (CompPlugin *p);
 
-typedef char **(*ListPluginsProc) (const char *path,
-				   int	      *n);
+typedef char **(*ListPluginsProc) (char *path,
+				   int	*n);
 
 extern LoadPluginProc   loaderLoadPlugin;
 extern UnloadPluginProc loaderUnloadPlugin;
@@ -2946,10 +2912,10 @@ void
 windowFiniPlugins (CompWindow *w);
 
 CompPlugin *
-findActivePlugin (const char *name);
+findActivePlugin (char *name);
 
 CompPlugin *
-loadPlugin (const char *plugin);
+loadPlugin (char *plugin);
 
 void
 unloadPlugin (CompPlugin *p);
@@ -2998,15 +2964,15 @@ destroyFunctionData (CompFunctionData *data);
 
 Bool
 addTempHeaderOpToFunctionData (CompFunctionData *data,
-			       const char	*name);
+			       char		*name);
 
 Bool
 addParamHeaderOpToFunctionData (CompFunctionData *data,
-				const char	 *name);
+				char		 *name);
 
 Bool
 addAttribHeaderOpToFunctionData (CompFunctionData *data,
-				 const char	  *name);
+				 char		  *name);
 
 #define COMP_FETCH_TARGET_2D   0
 #define COMP_FETCH_TARGET_RECT 1
@@ -3014,28 +2980,28 @@ addAttribHeaderOpToFunctionData (CompFunctionData *data,
 
 Bool
 addFetchOpToFunctionData (CompFunctionData *data,
-			  const char	   *dst,
-			  const char	   *offset,
+			  char		   *dst,
+			  char		   *offset,
 			  int		   target);
 
 Bool
 addColorOpToFunctionData (CompFunctionData *data,
-			  const char	   *dst,
-			  const char	   *src);
+			  char		   *dst,
+			  char		   *src);
 
 Bool
 addDataOpToFunctionData (CompFunctionData *data,
-			 const char	  *str,
+			 char		  *str,
 			 ...);
 
 Bool
 addBlendOpToFunctionData (CompFunctionData *data,
-			  const char	   *str,
+			  char		   *str,
 			  ...);
 
 int
 createFragmentFunction (CompScreen	 *s,
-			const char	 *name,
+			char		 *name,
 			CompFunctionData *data);
 
 void
