@@ -132,6 +132,8 @@ typedef struct _DecorScreen {
 typedef struct _DecorWindow {
     WindowDecoration *wd;
     Decoration	     *decor;
+
+    CompTimeoutHandle resizeUpdateHandle;
 } DecorWindow;
 
 #define GET_DECOR_DISPLAY(d)				      \
@@ -1115,7 +1117,7 @@ decorGetDisplayOptions (CompPlugin  *plugin,
 static Bool
 decorSetDisplayOption (CompPlugin      *plugin,
 		       CompDisplay     *display,
-		       const char      *name,
+		       char	       *name,
 		       CompOptionValue *value)
 {
     CompOption *o;
@@ -1202,6 +1204,20 @@ decorWindowMoveNotify (CompWindow *w,
     WRAP (ds, w->screen, windowMoveNotify, decorWindowMoveNotify);
 }
 
+static Bool
+decorResizeUpdateTimeout (void *closure)
+{
+    CompWindow *w = (CompWindow *) closure;
+
+    DECOR_WINDOW (w);
+
+    decorWindowUpdate (w, TRUE);
+
+    dw->resizeUpdateHandle = 0;
+
+    return FALSE;
+}
+
 static void
 decorWindowResizeNotify (CompWindow *w,
 			 int	    dx,
@@ -1210,9 +1226,16 @@ decorWindowResizeNotify (CompWindow *w,
 			 int	    dheight)
 {
     DECOR_SCREEN (w->screen);
+    DECOR_WINDOW (w);
 
-    if (!decorWindowUpdate (w, TRUE))
-	updateWindowDecorationScale (w);
+    /* FIXME: we should not need a timer for calling decorWindowUpdate, and only call
+       updateWindowDecorationScale if decorWindowUpdate returns FALSE. Unfortunately,
+       decorWindowUpdate may call updateWindowOutputExtents, which may call
+       WindowResizeNotify. As we never should call a wrapped function that's currently
+       processed, we need the timer for the moment. updateWindowOutputExtents should be
+       fixed so that it does not emit a resize notification. */
+    dw->resizeUpdateHandle = compAddTimeout (0, decorResizeUpdateTimeout, w);
+    updateWindowDecorationScale (w);
 
     UNWRAP (ds, w->screen, windowResizeNotify);
     (*w->screen->windowResizeNotify) (w, dx, dy, dwidth, dheight);
@@ -1220,7 +1243,8 @@ decorWindowResizeNotify (CompWindow *w,
 }
 
 static void
-decorWindowStateChangeNotify (CompWindow *w)
+decorWindowStateChangeNotify (CompWindow   *w,
+			      unsigned int lastState)
 {
     DECOR_SCREEN (w->screen);
     DECOR_WINDOW (w);
@@ -1237,7 +1261,7 @@ decorWindowStateChangeNotify (CompWindow *w)
     }
 
     UNWRAP (ds, w->screen, windowStateChangeNotify);
-    (*w->screen->windowStateChangeNotify) (w);
+    (*w->screen->windowStateChangeNotify) (w, lastState);
     WRAP (ds, w->screen, windowStateChangeNotify, decorWindowStateChangeNotify);
 }
 
@@ -1419,6 +1443,8 @@ decorInitWindow (CompPlugin *p,
     dw->wd    = NULL;
     dw->decor = NULL;
 
+    dw->resizeUpdateHandle = 0;
+
     w->privates[ds->windowPrivateIndex].ptr = dw;
 
     if (!w->attrib.override_redirect)
@@ -1439,6 +1465,9 @@ decorFiniWindow (CompPlugin *p,
     if (!w->destroyed)
 	decorWindowUpdate (w, FALSE);
 
+    if (dw->resizeUpdateHandle)
+	compRemoveTimeout (dw->resizeUpdateHandle);
+
     if (dw->wd)
 	destroyWindowDecoration (w->screen, dw->wd);
 
