@@ -153,7 +153,7 @@ placeGetScreenOptions (CompPlugin *plugin,
 static Bool
 placeSetScreenOption (CompPlugin      *plugin,
 		      CompScreen      *screen,
-		      const char      *name,
+		      char	      *name,
 		      CompOptionValue *value)
 {
     CompOption *o;
@@ -801,9 +801,18 @@ placeRandom (CompWindow *window,
 	     int	*x,
 	     int	*y)
 {
-    *x = workarea->x + rand () % (workarea->width - get_window_width (window));
-    *y = workarea->y + rand () % (workarea->height -
-				  get_window_height (window));
+    int remainX, remainY;
+
+    *x = workarea->x;
+    *y = workarea->y;
+
+    remainX = workarea->width - get_window_width (window);
+    if (remainX > 0)
+	*x += rand () % remainX;
+
+    remainY = workarea->height - get_window_height (window);
+    if (remainY > 0)
+	*y += rand () % remainY;
 }
 
 static void
