@@ -112,7 +112,7 @@ static CompMetadata wobblyMetadata;
 
 static int displayPrivateIndex;
 
-#define WOBBLY_DISPLAY_OPTION_SNAP_KEY      0
+#define WOBBLY_DISPLAY_OPTION_SNAP          0
 #define WOBBLY_DISPLAY_OPTION_SNAP_INVERTED 1
 #define WOBBLY_DISPLAY_OPTION_SHIVER        2
 #define WOBBLY_DISPLAY_OPTION_NUM           3
@@ -211,7 +211,7 @@ wobblyGetScreenOptions (CompPlugin *plugin,
 static Bool
 wobblySetScreenOption (CompPlugin      *plugin,
 		       CompScreen      *screen,
-		       const char      *name,
+		       char	       *name,
 		       CompOptionValue *value)
 {
     CompOption *o;
@@ -2125,8 +2125,7 @@ wobblyHandleEvent (CompDisplay *d,
 		Bool		    inverted;
 		unsigned int	    mods = 0xffffffff;
 
-		action   =
-		    &wd->opt[WOBBLY_DISPLAY_OPTION_SNAP_KEY].value.action;
+		action   = &wd->opt[WOBBLY_DISPLAY_OPTION_SNAP].value.action;
 		inverted = wd->opt[WOBBLY_DISPLAY_OPTION_SNAP_INVERTED].value.b;
 
 		if (action->type & CompBindingTypeKey)
@@ -2631,9 +2630,9 @@ wobblyGetDisplayOptions (CompPlugin  *plugin,
 }
 
 static Bool
-wobblySetDisplayOption (CompPlugin      *plugin,
+wobblySetDisplayOption (CompPlugin  *plugin,
 			CompDisplay     *display,
-			const char	*name,
+			char	        *name,
 			CompOptionValue *value)
 {
     CompOption *o;
@@ -2646,7 +2645,7 @@ wobblySetDisplayOption (CompPlugin      *plugin,
 	return FALSE;
 
     switch (index) {
-    case WOBBLY_DISPLAY_OPTION_SNAP_KEY:
+    case WOBBLY_DISPLAY_OPTION_SNAP:
 	/* ignore the key */
 	value->action.key.keycode = 0;
 
@@ -2661,10 +2660,10 @@ wobblySetDisplayOption (CompPlugin      *plugin,
 }
 
 static const CompMetadataOptionInfo wobblyDisplayOptionInfo[] = {
-    { "snap_key", "key", "<passive_grab>false</passive_grab>",
+    { "snap", "action", "<passive_grab>false</passive_grab>",
       wobblyEnableSnapping, wobblyDisableSnapping },
     { "snap_inverted", "bool", 0, 0, 0 },
-    { "shiver", "bell", 0, wobblyShiver, 0 }
+    { "shiver", "action", 0, wobblyShiver, 0 }
 };
 
 static Bool
