@@ -185,8 +185,10 @@ static Bool
 autoRaiseTimeout (void *closure)
 {
     CompDisplay *display = closure;
+    CompWindow *w = findWindowAtDisplay (display, display->activeWindow);
 
-    if (display->autoRaiseWindow == display->activeWindow)
+    if (display->autoRaiseWindow == display->activeWindow ||
+	display->autoRaiseWindow == w->transientFor)
     {
 	CompWindow *w;
 
@@ -206,7 +208,7 @@ isCallBackBinding (CompOption	   *option,
 		   CompBindingType type,
 		   CompActionState state)
 {
-    if (!isActionOption (option))
+    if (option->type != CompOptionTypeAction)
 	return FALSE;
 
     if (!(option->value.action.type & type))
@@ -312,14 +314,9 @@ triggerButtonPressBindings (CompDisplay *d,
 	    if (isInitiateBinding (option, CompBindingTypeEdgeButton,
 				   CompActionStateInitEdge, &action))
 	    {
-		if ((action->button.button == event->xbutton.button) &&
-		    (action->edgeMask & edge))
+		if (action->edgeMask & edge)
 		{
-		    bindMods = virtualToRealModMask (d,
-						     action->button.modifiers);
-
-		    if ((bindMods & modMask) ==
-			(event->xbutton.state & modMask))
+		    if (action->edgeButton == event->xbutton.button)
 			if ((*action->initiate) (d, action,
 						 CompActionStateInitEdge,
 						 argument, nArgument))
@@ -388,7 +385,7 @@ triggerKeyPressBindings (CompDisplay *d,
 
 	while (n--)
 	{
-	    if (isActionOption (o))
+	    if (o->type == CompOptionTypeAction)
 	    {
 		if (o->value.action.terminate)
 		    (*o->value.action.terminate) (d, &o->value.action,
@@ -539,8 +536,7 @@ isBellAction (CompOption      *option,
 	      CompActionState state,
 	      CompAction      **action)
 {
-    if (option->type != CompOptionTypeAction &&
-	option->type != CompOptionTypeBell)
+    if (option->type != CompOptionTypeAction)
 	return FALSE;
 
     if (!option->value.action.bell)
@@ -586,9 +582,7 @@ isEdgeAction (CompOption      *option,
 	      CompActionState state,
 	      unsigned int    edge)
 {
-    if (option->type != CompOptionTypeAction &&
-	option->type != CompOptionTypeButton &&
-	option->type != CompOptionTypeEdge)
+    if (option->type != CompOptionTypeAction)
 	return FALSE;
 
     if (!(option->value.action.edgeMask & edge))
@@ -1144,8 +1138,8 @@ handleActionEvent (CompDisplay *d,
 
 void
 handleCompizEvent (CompDisplay *d,
-		   const char  *pluginName,
-		   const char  *eventName,
+		   char        *pluginName,
+		   char        *eventName,
 		   CompOption  *option,
 		   int         nOption)
 {
@@ -1312,9 +1306,9 @@ handleEvent (CompDisplay *d,
 		if (w->state & CompWindowStateHiddenMask)
 		{
 		    w->minimized = FALSE;
-		    w->state &= ~CompWindowStateHiddenMask;
 
-		    changeWindowState (w, w->state);
+		    changeWindowState (w,
+				       w->state & ~CompWindowStateHiddenMask);
 
 		    updateClientListForScreen (w->screen);
 		}
@@ -1598,7 +1592,12 @@ handleEvent (CompDisplay *d,
 	{
 	    w = findWindowAtDisplay (d, event->xclient.window);
 	    if (w)
-		activateWindow (w);
+	    {
+		/* use focus stealing prevention if request came from an
+		   application (which means data.l[0] is 1 */
+		if (event->xclient.data.l[0] != 1 || allowWindowFocus (w))
+		    activateWindow (w);
+	    }
 	}
 	else if (event->xclient.message_type == d->winOpacityAtom)
 	{
@@ -1682,13 +1681,11 @@ handleEvent (CompDisplay *d,
 				  CompWindowStateMaximizedVertMask))
 			stackingUpdateMode = CompStackingUpdateModeNormal;
 
-		    w->state = wState;
+		    changeWindowState (w, wState);
 
 		    recalcWindowType (w);
 		    recalcWindowActions (w);
 
-		    changeWindowState (w, w->state);
-
 		    updateWindowAttributes (w, stackingUpdateMode);
 		}
 	    }
@@ -1872,15 +1869,33 @@ handleEvent (CompDisplay *d,
 	w = findWindowAtDisplay (d, event->xmaprequest.window);
 	if (w)
 	{
-	    if (w->minimized)
-		unminimizeWindow (w);
+	    XWindowAttributes attr;
+	    Bool              doMapProcessing = TRUE;
 
-	    (*w->screen->leaveShowDesktopMode) (w->screen, w);
+	    /* We should check the override_redirect flag here, because the
+	       client might have changed it while being unmapped. */
+	    if (XGetWindowAttributes (d->display, w->id, &attr))
+	    {
+		if (w->attrib.override_redirect != attr.override_redirect)
+		{
+		    w->attrib.override_redirect = attr.override_redirect;
+		    recalcWindowType (w);
+		    recalcWindowActions (w);
+
+		    (*d->matchPropertyChanged) (d, w);
+		}
+	    }
 
 	    w->managed = TRUE;
 
-	    if (!(w->state & CompWindowStateHiddenMask))
+	    if (w->state & CompWindowStateHiddenMask)
+		if (!w->minimized && !w->inShowDesktopMode)
+		    doMapProcessing = FALSE;
+
+	    if (doMapProcessing)
 	    {
+		Bool allowFocus;
+
 		w->initialViewportX = w->screen->x;
 		w->initialViewportY = w->screen->y;
 
@@ -1905,17 +1920,16 @@ handleEvent (CompDisplay *d,
 		    w->placed   = TRUE;
 		}
 
-		w->pendingMaps++;
+		allowFocus = allowWindowFocus (w);
 
-		XMapWindow (d->display, event->xmaprequest.window);
+		if (allowFocus)
+		    updateWindowAttributes (w, CompStackingUpdateModeNormal);
+		else
+		    updateWindowAttributes (w, CompStackingUpdateModeNone);
 
-		updateWindowAttributes (w, CompStackingUpdateModeNormal);
-
-		if (focusWindowOnMap (w))
-		{
-		    moveInputFocusToWindow (w);
-		}
-		else if (w->type & ~CompWindowTypeSplashMask)
+		if (!allowFocus &&
+		    (w->type & ~(CompWindowTypeSplashMask |
+				 CompWindowTypeDockMask)))
 		{
 		    CompWindow *p;
 
@@ -1927,6 +1941,20 @@ handleEvent (CompDisplay *d,
 		    if (p)
 			restackWindowBelow (w, p);
 		}
+
+		if (w->minimized)
+		    unminimizeWindow (w);
+
+		(*w->screen->leaveShowDesktopMode) (w->screen, w);
+
+		if (!(w->state & CompWindowStateHiddenMask))
+		{
+		    w->pendingMaps++;
+		    XMapWindow (d->display, w->id);
+		}
+
+		if (allowFocus)
+		    moveInputFocusToWindow (w);
 	    }
 
 	    setWindowProp (d, w->id, d->winDesktopAtom, w->desktop);
@@ -1951,6 +1979,40 @@ handleEvent (CompDisplay *d,
 	    xwc.border_width = event->xconfigurerequest.border_width;
 
 	    moveResizeWindow (w, &xwc, event->xconfigurerequest.value_mask, 0);
+
+	    if (event->xconfigurerequest.value_mask & CWStackMode)
+	    {
+		Window     above = event->xconfigure.above;
+		CompWindow *sibling = findWindowAtDisplay (d, above);
+
+		switch (event->xconfigurerequest.detail) {
+		case Above:
+		    if (allowWindowFocus (w))
+		    {
+			if (above)
+			{
+			    if (sibling)
+				restackWindowAbove (w, sibling);
+			}
+			else
+			    raiseWindow (w);
+		    }
+		    break;
+		case Below:
+		    if (above)
+		    {
+			if (sibling)
+			    restackWindowBelow (w, sibling);
+		    }
+		    else
+			lowerWindow (w);
+		    break;
+		default:
+		    /* no handling of the TopIf, BottomIf, Opposite cases -
+		       there will hardly be any client using that */
+		    break;
+		}
+	    }
 	}
 	else
 	{
