@@ -417,7 +417,7 @@ compGetScreenOptions (CompScreen *screen,
 
 static Bool
 setScreenOption (CompScreen      *screen,
-		 const char	 *name,
+		 char	         *name,
 		 CompOptionValue *value)
 {
     CompOption *o;
@@ -552,8 +552,8 @@ setScreenOption (CompScreen      *screen,
 
 static Bool
 setScreenOptionForPlugin (CompScreen      *screen,
-			  const char	  *plugin,
-			  const char	  *name,
+			  char	          *plugin,
+			  char	          *name,
 			  CompOptionValue *value)
 {
     CompPlugin *p;
@@ -800,6 +800,17 @@ reshape (CompScreen *s,
 	XMoveResizeWindow (s->display->display, s->overlay, 0, 0, w, h);
 #endif
 
+    if (s->display->xineramaExtension)
+    {
+	CompDisplay *d = s->display;
+
+	if (d->screenInfo)
+	    XFree (d->screenInfo);
+
+	d->nScreenInfo = 0;
+	d->screenInfo = XineramaQueryScreens (d->display, &d->nScreenInfo);
+    }
+
     glMatrixMode (GL_PROJECTION);
     glLoadIdentity ();
     glMatrixMode (GL_MODELVIEW);
@@ -1642,6 +1653,7 @@ addScreen (CompDisplay *display,
     {
 	compLogMessage (display, "core", CompLogLevelFatal,
 			"Couldn't allocate color");
+	XFree (visinfo);
 	return FALSE;
     }
 
@@ -1650,6 +1662,7 @@ addScreen (CompDisplay *display,
     {
 	compLogMessage (display, "core", CompLogLevelFatal,
 			"Couldn't create bitmap");
+	XFree (visinfo);
 	return FALSE;
     }
 
@@ -1659,6 +1672,7 @@ addScreen (CompDisplay *display,
     {
 	compLogMessage (display, "core", CompLogLevelFatal,
 			"Couldn't create invisible cursor");
+	XFree (visinfo);
 	return FALSE;
     }
 
@@ -1670,6 +1684,7 @@ addScreen (CompDisplay *display,
     {
 	compLogMessage (display, "core", CompLogLevelFatal,
 			"Root visual is not a GL visual");
+	XFree (visinfo);
 	return FALSE;
     }
 
@@ -1678,6 +1693,7 @@ addScreen (CompDisplay *display,
     {
 	compLogMessage (display, "core", CompLogLevelFatal,
 			"Root visual is not a double buffered GL visual");
+	XFree (visinfo);
 	return FALSE;
     }
 
